@@ -42,6 +42,7 @@
 
 #include <libgdict/gdict.h>
 
+#include "gdict-sidebar.h"
 #include "gdict-print.h"
 #include "gdict-pref-dialog.h"
 #include "gdict-about.h"
@@ -54,10 +55,18 @@
 #define GDICT_WINDOW_MIN_WIDTH	  400
 #define GDICT_WINDOW_MIN_HEIGHT	  330
 
+enum
+{
+  COMPLETION_TEXT_COLUMN,
+
+  COMPLETION_N_COLUMNS
+};
+
 enum
 {
   PROP_0,
-  
+
+  PROP_ACTION,
   PROP_SOURCE_LOADER,
   PROP_SOURCE_NAME,
   PROP_PRINT_FONT,
@@ -121,6 +130,12 @@ gdict_window_finalize (GObject *object)
   
   g_object_unref (window->action_group);
 
+  if (window->completion)
+    g_object_unref (window->completion);
+
+  if (window->completion_model)
+    g_object_unref (window->completion_model);
+
   if (window->busy_cursor)
     gdk_cursor_unref (window->busy_cursor);
 
@@ -135,19 +150,36 @@ gdict_window_finalize (GObject *object)
 }
 
 static void
-gdict_window_set_speller_visible (GdictWindow *window,
+gdict_window_set_sidebar_visible (GdictWindow *window,
 				  gboolean     is_visible)
 {
   g_assert (GDICT_IS_WINDOW (window));
   
-  if (is_visible != window->speller_visible)
+  if (is_visible != window->sidebar_visible)
     {
       if (is_visible)
-	gtk_widget_show (window->speller_frame);
+	gtk_widget_show (window->sidebar_frame);
       else
-	gtk_widget_hide (window->speller_frame);
+	gtk_widget_hide (window->sidebar_frame);
 
-      window->speller_visible = is_visible;
+      window->sidebar_visible = is_visible;
+    }
+}
+
+static void
+gdict_window_set_statusbar_visible (GdictWindow *window,
+				    gboolean     is_visible)
+{
+  g_assert (GDICT_IS_WINDOW (window));
+
+  if (is_visible != window->statusbar_visible)
+    {
+      if (is_visible)
+	gtk_widget_show (window->status);
+      else
+	gtk_widget_hide (window->status);
+
+      window->statusbar_visible = is_visible;
     }
 }
 
@@ -186,8 +218,6 @@ gdict_window_lookup_start_cb (GdictContext *context,
   if (!window->busy_cursor)
     window->busy_cursor = gdk_cursor_new (GDK_WATCH);
 
-  gdict_window_set_speller_visible (window, FALSE);
-  
   message = g_strdup_printf (_("Searching for '%s'..."), window->word);
   
   if (window->status)
@@ -211,6 +241,7 @@ gdict_window_lookup_end_cb (GdictContext *context,
 {
   gchar *message;
   gint count;
+  GtkTreeIter iter;
 
   count = window->current_definition;
 
@@ -230,6 +261,11 @@ gdict_window_lookup_end_cb (GdictContext *context,
   if (window->progress)
     gtk_widget_hide (window->progress);
 
+  gtk_list_store_append (window->completion_model, &iter);
+  gtk_list_store_set (window->completion_model, &iter,
+		      COMPLETION_TEXT_COLUMN, window->word,
+		      -1);
+
   gdk_window_set_cursor (GTK_WIDGET (window)->window, NULL);
   g_free (message);
 }
@@ -253,9 +289,9 @@ gdict_window_error_cb (GdictContext *context,
     {
       GdictSource *source;
       GdictContext *context;
-      
-      gtk_widget_show (window->speller_frame);
-      window->speller_visible = TRUE;
+
+      gdict_window_set_sidebar_visible (window, TRUE);
+      gdict_sidebar_view_page (GDICT_SIDEBAR (window->sidebar), "speller");
 
       /* we clone the context, so that the signals that it
        * fires do not get caught by the signal handlers we
@@ -282,21 +318,18 @@ static void
 gdict_window_set_database (GdictWindow *window,
 			   const gchar *database)
 {
-  if (window->database)
-    g_free (window->database);
+  g_free (window->database);
 
-  if (!database)
-    database = gconf_client_get_string (window->gconf_client,
-		    			GDICT_GCONF_DATABASE_KEY,
-					NULL);
-  
-  if (!database)
-    database = GDICT_DEFAULT_DATABASE;
-
-  window->database = g_strdup (database);
+  if (database)
+    window->database = g_strdup (database);
+  else
+    window->database = gdict_gconf_get_string_with_default (window->gconf_client,
+							    GDICT_GCONF_DATABASE_KEY,
+							    GDICT_DEFAULT_DATABASE);
 
   if (window->defbox)
-    gdict_defbox_set_database (GDICT_DEFBOX (window->defbox), database);
+    gdict_defbox_set_database (GDICT_DEFBOX (window->defbox),
+			       window->database);
 }
 
 static void
@@ -366,42 +399,32 @@ static void
 gdict_window_set_defbox_font (GdictWindow *window,
 			      const gchar *defbox_font)
 {
-  if (!defbox_font)
-    defbox_font = gconf_client_get_string (window->gconf_client,
-		    			   DOCUMENT_FONT_KEY,
-					   NULL);
+  g_free (window->defbox_font);
 
-  if (!defbox_font)
-    defbox_font = GDICT_DEFAULT_DEFBOX_FONT;
+  if (defbox_font)
+    window->defbox_font = g_strdup (defbox_font);
+  else
+    window->defbox_font = gdict_gconf_get_string_with_default (window->gconf_client,
+							       DOCUMENT_FONT_KEY,
+							       GDICT_DEFAULT_DEFBOX_FONT);
 
-  if (window->defbox_font)
-    g_free (window->defbox_font);
-
-  window->defbox_font = g_strdup (defbox_font);
-
-  if (!window->defbox)
-    return;
-
-  gdict_defbox_set_font_name (GDICT_DEFBOX (window->defbox),
-		  	      window->defbox_font);
+  if (window->defbox)
+    gdict_defbox_set_font_name (GDICT_DEFBOX (window->defbox),
+				window->defbox_font);
 }
 
 static void
 gdict_window_set_print_font (GdictWindow *window,
 			     const gchar *print_font)
 {
-  if (!print_font)
-    print_font = gconf_client_get_string (window->gconf_client,
-    					  GDICT_GCONF_PRINT_FONT_KEY,
-    					  NULL);
-  
-  if (!print_font)
-    print_font = GDICT_DEFAULT_PRINT_FONT;
-  
-  if (window->print_font)
-    g_free (window->print_font);
-  
-  window->print_font = g_strdup (print_font);
+  g_free (window->print_font);
+
+  if (print_font)
+    window->print_font = g_strdup (print_font);
+  else
+    window->print_font = gdict_gconf_get_string_with_default (window->gconf_client,
+							      GDICT_GCONF_PRINT_FONT_KEY,
+							      GDICT_DEFAULT_PRINT_FONT);
 }
 
 static void
@@ -485,23 +508,17 @@ gdict_window_set_source_name (GdictWindow *window,
 			      const gchar *source_name)
 {
   GdictContext *context;
-  
-  /* some sanity checks first */
-  if (!source_name)
-    source_name = gconf_client_get_string (window->gconf_client,
-      					   GDICT_GCONF_SOURCE_KEY,
-      					   NULL);
-  
-  if (!source_name)
-    source_name = GDICT_DEFAULT_SOURCE_NAME;
-  
-  if (window->source_name)
-    g_free (window->source_name);
-  
-  window->source_name = g_strdup (source_name);
-  
+
+  g_free (window->source_name);
+
+  if (source_name)
+    window->source_name = g_strdup (source_name);
+  else
+    window->source_name = gdict_gconf_get_string_with_default (window->gconf_client,
+							       GDICT_GCONF_SOURCE_KEY,
+							       GDICT_DEFAULT_SOURCE_NAME);
+
   context = get_context_from_loader (window);
-  
   gdict_window_set_context (window, context);
 }
 
@@ -515,6 +532,9 @@ gdict_window_set_property (GObject      *object,
   
   switch (prop_id)
     {
+    case PROP_ACTION:
+      window->action = g_value_get_enum (value);
+      break;
     case PROP_SOURCE_LOADER:
       if (window->loader)
         g_object_unref (window->loader);
@@ -549,6 +569,9 @@ gdict_window_get_property (GObject    *object,
   
   switch (prop_id)
     {
+    case PROP_ACTION:
+      g_value_set_enum (value, window->action);
+      break;
     case PROP_SOURCE_LOADER:
       g_value_set_object (value, window->loader);
       break;
@@ -595,11 +618,25 @@ gdict_window_cmd_file_new (GtkAction   *action,
 			 window->is_maximized,
 			 NULL);
   gconf_client_set_bool (window->gconf_client,
-		  	 GDICT_GCONF_SPELLER_VISIBLE_KEY,
-			 window->speller_visible,
+		  	 GDICT_GCONF_SIDEBAR_VISIBLE_KEY,
+			 window->sidebar_visible,
+			 NULL);
+  gconf_client_set_int (window->gconf_client,
+		  	GDICT_GCONF_SIDEBAR_WIDTH_KEY,
+			window->sidebar_width,
+			NULL);
+  gconf_client_set_string (window->gconf_client,
+		  	   GDICT_GCONF_SIDEBAR_PAGE_KEY,
+			   gdict_sidebar_current_page (GDICT_SIDEBAR (window->sidebar)),
+			   NULL);
+  gconf_client_set_bool (window->gconf_client,
+		  	 GDICT_GCONF_STATUSBAR_VISIBLE_KEY,
+			 window->statusbar_visible,
 			 NULL);
  
-  new_window = gdict_window_new (window->loader, NULL, NULL);
+  new_window = gdict_window_new (GDICT_WINDOW_ACTION_CLEAR,
+  				 window->loader,
+				 NULL, NULL);
   gtk_widget_show (new_window);
   
   g_signal_emit (window, gdict_window_signals[CREATED], 0, new_window);
@@ -691,8 +728,20 @@ gdict_window_cmd_file_close_window (GtkAction   *action,
 			 window->is_maximized,
 			 NULL);
   gconf_client_set_bool (window->gconf_client,
-		         GDICT_GCONF_SPELLER_VISIBLE_KEY,
-			 window->speller_visible,
+		         GDICT_GCONF_SIDEBAR_VISIBLE_KEY,
+			 window->sidebar_visible,
+			 NULL);
+  gconf_client_set_int  (window->gconf_client,
+		  	 GDICT_GCONF_SIDEBAR_WIDTH_KEY,
+			 window->sidebar_width,
+			 NULL);
+  gconf_client_set_string (window->gconf_client,
+		           GDICT_GCONF_SIDEBAR_PAGE_KEY,
+			   gdict_sidebar_current_page (GDICT_SIDEBAR (window->sidebar)),
+			   NULL);
+  gconf_client_set_bool (window->gconf_client,
+		  	 GDICT_GCONF_STATUSBAR_VISIBLE_KEY,
+			 window->statusbar_visible,
 			 NULL);
   
   /* if this was called from the uimanager, destroy the widget;
@@ -761,15 +810,49 @@ gdict_window_cmd_edit_preferences (GtkAction   *action,
 }
 
 static void
-gdict_window_cmd_view_speller (GtkAction   *action,
+gdict_window_cmd_view_sidebar (GtkAction   *action,
 			       GdictWindow *window)
 {
   gboolean is_active;
   
   g_assert (GDICT_IS_WINDOW (window));
 
-  is_active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
-  gdict_window_set_speller_visible (window, is_active);
+  if (window->sidebar_visible)
+    gdict_window_set_sidebar_visible (window, FALSE);
+  else
+    gdict_window_set_sidebar_visible (window, TRUE);
+}
+
+static void
+gdict_window_cmd_view_statusbar (GtkAction   *action,
+				 GdictWindow *window)
+{
+  g_assert (GDICT_IS_WINDOW (window));
+
+  if (window->statusbar_visible)
+    gdict_window_set_statusbar_visible (window, FALSE);
+  else
+    gdict_window_set_statusbar_visible (window, TRUE);
+}
+
+static void
+gdict_window_cmd_view_speller (GtkAction   *action,
+			       GdictWindow *window)
+{
+  g_assert (GDICT_IS_WINDOW (window));
+
+  gdict_sidebar_view_page (GDICT_SIDEBAR (window->sidebar), "speller");
+  gdict_window_set_sidebar_visible (window, TRUE);
+}
+
+static void
+gdict_window_cmd_view_databases (GtkAction   *action,
+				 GdictWindow *window)
+{
+  g_assert (GDICT_IS_WINDOW (window));
+
+  gdict_sidebar_view_page (GDICT_SIDEBAR (window->sidebar), "databases");
+  gdict_window_set_sidebar_visible (window, TRUE);
 }
 
 static void
@@ -877,13 +960,6 @@ gdict_window_cmd_escape (GtkAction   *action,
   gdict_defbox_set_show_find (GDICT_DEFBOX (window->defbox), FALSE);
 }
 
-static const GtkToggleActionEntry toggle_entries[] =
-{
-  { "ViewSpeller", NULL, N_("_Similar Words"), "<control>T",
-    N_("Show similar words"),
-    G_CALLBACK (gdict_window_cmd_view_speller), FALSE },
-};
-
 static const GtkActionEntry entries[] =
 {
   { "File", NULL, N_("_File") },
@@ -918,6 +994,14 @@ static const GtkActionEntry entries[] =
     G_CALLBACK (gdict_window_cmd_edit_preferences) },
 
   /* View menu */
+  { "ViewSidebar", NULL, N_("_Sidebar"), "F9", NULL,
+    G_CALLBACK (gdict_window_cmd_view_sidebar) },
+  { "ViewStatusbar", NULL, N_("S_tatusbar"), NULL, NULL,
+    G_CALLBACK (gdict_window_cmd_view_statusbar) },
+  { "ViewSpeller", NULL, N_("Similar _Words"), "<control>T", NULL,
+    G_CALLBACK (gdict_window_cmd_view_speller) },
+  { "ViewDB", NULL, N_("Available _Databases"), "<control>B", NULL,
+    G_CALLBACK (gdict_window_cmd_view_databases) },
 
   /* Go menu */
   { "GoPreviousDef", GTK_STOCK_GO_BACK, N_("_Previous Definition"), "<control>Page_Up",
@@ -1004,12 +1088,21 @@ gdict_window_gconf_notify_cb (GConfClient *client,
       else
         gdict_window_set_defbox_font (window, GDICT_DEFAULT_DEFBOX_FONT);
     }
-  else if (strcmp (entry->key, GDICT_GCONF_SPELLER_VISIBLE_KEY) == 0)
+  else if (strcmp (entry->key, GDICT_GCONF_SIDEBAR_VISIBLE_KEY) == 0)
     {
       if (entry->value && (entry->value->type == GCONF_VALUE_BOOL))
-        gdict_window_set_speller_visible (window, gconf_value_get_bool (entry->value));
+        gdict_window_set_sidebar_visible (window,
+					  gconf_value_get_bool (entry->value));
       else
-        gdict_window_set_speller_visible (window, FALSE);
+        gdict_window_set_sidebar_visible (window, FALSE);
+    }
+  else if (strcmp (entry->key, GDICT_GCONF_STATUSBAR_VISIBLE_KEY) == 0)
+    {
+      if (entry->value && (entry->value->type == GCONF_VALUE_BOOL))
+	gdict_window_set_statusbar_visible (window,
+					    gconf_value_get_bool (entry->value));
+      else
+	gdict_window_set_statusbar_visible (window, FALSE);
     }
 }
 
@@ -1032,6 +1125,24 @@ entry_activate_cb (GtkWidget   *widget,
   gdict_window_set_word (window, word, NULL);
 }
 
+static void
+database_activated_cb (GdictDatabaseChooser *chooser,
+		       const gchar          *db_name,
+		       const gchar          *db_desc,
+		       GdictWindow          *window)
+{
+  gdict_window_set_database (window, db_name);
+
+  if (window->status)
+    {
+      gchar *message;
+
+      message = g_strdup_printf (_("Database `%s' selected"), db_desc);
+      gtk_statusbar_push (GTK_STATUSBAR (window->status), 0, message);
+      g_free (message);
+    }
+}
+
 static void
 speller_word_activated_cb (GdictSpeller *speller,
 			   const gchar  *word,
@@ -1041,14 +1152,49 @@ speller_word_activated_cb (GdictSpeller *speller,
   gtk_entry_set_text (GTK_ENTRY (window->entry), word);
   
   gdict_window_set_word (window, word, db_name);
+
+  if (window->status)
+    {
+      gchar *message;
+
+      message = g_strdup_printf (_("Word `%s' selected"), word);
+      gtk_statusbar_push (GTK_STATUSBAR (window->status), 0, message);
+      g_free (message);
+    }
 }
 
 static void
-speller_closed_cb (GdictSpeller *speller,
+sidebar_page_changed_cb (GdictSidebar *sidebar,
+			 GdictWindow  *window)
+{
+  const gchar *page_id;
+  const gchar *message;
+
+  page_id = gdict_sidebar_current_page (sidebar);
+
+  switch (page_id[0])
+    {
+    case 's': /* speller */
+      message = _("Double-click on the word to look up");
+      break;
+    case 'd': /* db-chooser */
+      message = _("Double-click on the database to use");
+      break;
+    default:
+      message = NULL;
+      break;
+    }
+
+  if (message && window->status)
+    gtk_statusbar_push (GTK_STATUSBAR (window->status), 0, message);
+}
+
+static void
+sidebar_closed_cb (GdictSidebar *sidebar,
 		   GdictWindow  *window)
 {
-  gtk_widget_hide (window->speller_frame);
-  window->speller_visible = FALSE;
+  gtk_widget_hide (window->sidebar_frame);
+  window->sidebar_visible = FALSE;
 }
 
 static void
@@ -1180,6 +1326,20 @@ gdict_window_style_set (GtkWidget *widget,
   set_window_default_size (GDICT_WINDOW (widget));
 }
 
+static void
+gdict_window_handle_notify_position_cb (GtkWidget  *widget,
+					GParamSpec *pspec,
+					gpointer    user_data)
+{
+  GdictWindow *window = GDICT_WINDOW (user_data);
+  gint window_width, pos;
+
+  pos = gtk_paned_get_position (GTK_PANED (widget));
+  window_width = GTK_WIDGET (window)->allocation.width;
+
+  window->sidebar_width = window_width - pos;
+}
+
 static GObject *
 gdict_window_constructor (GType                  type,
 			  guint                  n_construct_properties,
@@ -1187,7 +1347,7 @@ gdict_window_constructor (GType                  type,
 {
   GObject *object;
   GdictWindow *window;
-  gint width, height;
+  gint width, height, sidebar_width;
   gboolean is_maximized;
   GtkWidget *hbox;
   GtkWidget *handle;
@@ -1198,9 +1358,10 @@ gdict_window_constructor (GType                  type,
   GtkAccelGroup *accel_group;
   GtkAction *action;
   PangoFontDescription *font_desc;
-  gchar *font_name;
+  gchar *font_name, *sidebar_page;
   GError *error;
-  gboolean speller_visible;
+  gboolean sidebar_visible;
+  gboolean statusbar_visible;
   
   object = G_OBJECT_CLASS (gdict_window_parent_class)->constructor (type,
   						   n_construct_properties,
@@ -1220,9 +1381,6 @@ gdict_window_constructor (GType                  type,
   gtk_action_group_add_actions (action_group, entries,
   				G_N_ELEMENTS (entries),
   				window);
-  gtk_action_group_add_toggle_actions (action_group, toggle_entries,
-		                       G_N_ELEMENTS (toggle_entries),
-				       window);
   
   window->ui_manager = gtk_ui_manager_new ();
   gtk_ui_manager_insert_action_group (window->ui_manager, action_group, 0);
@@ -1258,11 +1416,23 @@ gdict_window_constructor (GType                  type,
   label = gtk_label_new_with_mnemonic (_("Look _up:"));
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
   gtk_widget_show (label);
+
+  window->completion_model = gtk_list_store_new (COMPLETION_N_COLUMNS,
+		  				 G_TYPE_STRING);
+  
+  window->completion = gtk_entry_completion_new ();
+  gtk_entry_completion_set_popup_completion (window->completion, TRUE);
+  gtk_entry_completion_set_model (window->completion,
+		  		  GTK_TREE_MODEL (window->completion_model));
+  gtk_entry_completion_set_text_column (window->completion,
+		  			COMPLETION_TEXT_COLUMN);
   
   window->entry = gtk_entry_new ();
   if (window->word)
     gtk_entry_set_text (GTK_ENTRY (window->entry), window->word);
   
+  gtk_entry_set_completion (GTK_ENTRY (window->entry),
+		  	    window->completion);
   g_signal_connect (window->entry, "activate", G_CALLBACK (entry_activate_cb), window);
   gtk_box_pack_start (GTK_BOX (hbox), window->entry, TRUE, TRUE, 0);
   gtk_widget_show (window->entry);
@@ -1273,11 +1443,8 @@ gdict_window_constructor (GType                  type,
   gtk_box_pack_start (GTK_BOX (vbox), handle, TRUE, TRUE, 0);
   gtk_widget_show (handle);
 
-  frame1 = gtk_frame_new (NULL);
-  frame2 = gtk_frame_new (NULL);
-  gtk_frame_set_shadow_type (GTK_FRAME (frame1), GTK_SHADOW_IN);
-  gtk_frame_set_shadow_type (GTK_FRAME (frame2), GTK_SHADOW_IN);
-  gtk_widget_set_size_request (frame2, 150, -1);
+  frame1 = gtk_vbox_new (FALSE, 0);
+  frame2 = gtk_vbox_new (FALSE, 0);
   
   window->defbox = gdict_defbox_new ();
   if (window->context)
@@ -1293,57 +1460,77 @@ gdict_window_constructor (GType                  type,
   gtk_container_add (GTK_CONTAINER (frame1), window->defbox);
   gtk_widget_show (window->defbox);
 
+  window->sidebar = gdict_sidebar_new ();
+  g_signal_connect (window->sidebar, "page-changed",
+		    G_CALLBACK (sidebar_page_changed_cb),
+		    window);
+  g_signal_connect (window->sidebar, "closed",
+		    G_CALLBACK (sidebar_closed_cb),
+		    window);
+  
   window->speller = gdict_speller_new ();
   if (window->context)
-    gdict_speller_set_context (GDICT_SPELLER (window->speller), window->context);
-
+    gdict_speller_set_context (GDICT_SPELLER (window->speller),
+		    	       window->context);
   g_signal_connect (window->speller, "word-activated",
 		    G_CALLBACK (speller_word_activated_cb),
 		    window);
-  g_signal_connect (window->speller, "closed",
-		    G_CALLBACK (speller_closed_cb),
-		    window);
-  gtk_container_add (GTK_CONTAINER (frame2), window->speller);
+  
+  gdict_sidebar_add_page (GDICT_SIDEBAR (window->sidebar),
+		          "speller",
+			  _("Similar words"),
+			  window->speller);
   gtk_widget_show (window->speller);
 
+  window->db_chooser = gdict_database_chooser_new ();
+  if (window->context)
+    gdict_database_chooser_set_context (GDICT_DATABASE_CHOOSER (window->db_chooser),
+			    		window->context);
+  g_signal_connect (window->db_chooser, "database-activated",
+	  	    G_CALLBACK (database_activated_cb),
+		    window);
+  gdict_sidebar_add_page (GDICT_SIDEBAR (window->sidebar),
+	  		  "db-chooser",
+			  _("Available dictionaries"),
+			  window->db_chooser);
+  gtk_widget_show (window->db_chooser);
+  
+  gtk_container_add (GTK_CONTAINER (frame2), window->sidebar);
+  gtk_widget_show (window->sidebar);
+
   gtk_paned_pack1 (GTK_PANED (handle), frame1, TRUE, FALSE);
   gtk_paned_pack2 (GTK_PANED (handle), frame2, FALSE, TRUE);
 
   window->defbox_frame = frame1;
-  window->speller_frame = frame2;
+  window->sidebar_frame = frame2;
 
   gtk_widget_show (window->defbox_frame);
 
   window->status = gtk_statusbar_new ();
   gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (window->status), TRUE);
   gtk_box_pack_end (GTK_BOX (window->main_box), window->status, FALSE, FALSE, 0);
-  gtk_widget_show (window->status);
+  statusbar_visible = gconf_client_get_bool (window->gconf_client,
+		  			     GDICT_GCONF_STATUSBAR_VISIBLE_KEY,
+					     NULL);
+  gdict_window_set_statusbar_visible (window, statusbar_visible);
 
   window->progress = gtk_progress_bar_new ();
   gtk_box_pack_end (GTK_BOX (window->status), window->progress, FALSE, FALSE, 0);
 
   /* retrieve the font size from gconf */
-  font_name = gconf_client_get_string (window->gconf_client,
-		  		       DOCUMENT_FONT_KEY,
-				       NULL);
-  if (!font_name)
-    font_name = GDICT_DEFAULT_DEFBOX_FONT;
-  
+  font_name = gdict_gconf_get_string_with_default (window->gconf_client,
+						   DOCUMENT_FONT_KEY,
+						   GDICT_DEFAULT_DEFBOX_FONT);
+
   gdict_window_set_defbox_font (window, font_name);
   font_desc = pango_font_description_from_string (font_name);
+  g_free (font_name);
 
-  speller_visible = gconf_client_get_bool (window->gconf_client,
-		  			   GDICT_GCONF_SPELLER_VISIBLE_KEY,
+  sidebar_visible = gconf_client_get_bool (window->gconf_client,
+		  			   GDICT_GCONF_SIDEBAR_VISIBLE_KEY,
 					   NULL);
-  action = gtk_action_group_get_action (action_group, "ViewSpeller");
-  if (action)
-    {
-      gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action),
-		                    speller_visible);
-      gtk_toggle_action_toggled (GTK_TOGGLE_ACTION (action));
-    }
-  window->speller_visible = speller_visible;
-  
+  gdict_window_set_sidebar_visible (window, sidebar_visible);
+
   /* retrieve the window state from gconf */
   is_maximized = gconf_client_get_bool (window->gconf_client,
 		  			GDICT_GCONF_WINDOW_IS_MAXIMIZED_KEY,
@@ -1355,7 +1542,13 @@ gdict_window_constructor (GType                  type,
   height = gconf_client_get_int (window->gconf_client,
 		  		 GDICT_GCONF_WINDOW_HEIGHT_KEY,
 				 NULL);
-  
+  sidebar_width = gconf_client_get_int (window->gconf_client,
+		  			GDICT_GCONF_SIDEBAR_WIDTH_KEY,
+					NULL);
+  sidebar_page = gconf_client_get_string (window->gconf_client,
+		  			  GDICT_GCONF_SIDEBAR_PAGE_KEY,
+					  NULL);
+
   /* if the (width, height) tuple is not defined, use the font to
    * calculate the right window geometry
    */
@@ -1385,13 +1578,22 @@ gdict_window_constructor (GType                  type,
   			       height);
   if (is_maximized)
     gtk_window_maximize (GTK_WINDOW (window));
-  
+
+  gtk_paned_set_position (GTK_PANED (handle),
+		          GTK_WIDGET (window)->allocation.width - sidebar_width);
+  gdict_sidebar_view_page (GDICT_SIDEBAR (window->sidebar), sidebar_page);
+
+  g_free (sidebar_page);
+
   g_signal_connect (window, "delete-event",
 		    G_CALLBACK (gdict_window_delete_event_cb),
 		    NULL);
   g_signal_connect (window, "window-state-event",
 		    G_CALLBACK (gdict_window_state_event_cb),
 		    NULL);
+  g_signal_connect (handle, "notify::position",
+		    G_CALLBACK (gdict_window_handle_notify_position_cb),
+		    window);
 
   gtk_widget_pop_composite_child ();
   
@@ -1412,6 +1614,14 @@ gdict_window_class_init (GdictWindowClass *klass)
   widget_class->style_set = gdict_window_style_set;
   widget_class->size_allocate = gdict_window_size_allocate;
   
+  g_object_class_install_property (gobject_class,
+  				   PROP_ACTION,
+				   g_param_spec_enum ("action",
+				   		      "Action",
+						      "The default action performed by the window",
+						      GDICT_TYPE_WINDOW_ACTION,
+						      GDICT_WINDOW_ACTION_CLEAR,
+						      (G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY)));
   g_object_class_install_property (gobject_class,
   				   PROP_SOURCE_LOADER,
   				   g_param_spec_object ("source-loader",
@@ -1472,6 +1682,8 @@ static void
 gdict_window_init (GdictWindow *window)
 {
   GError *gconf_error;
+
+  window->action = GDICT_WINDOW_ACTION_CLEAR;
   
   window->loader = NULL;
   window->context = NULL;
@@ -1532,7 +1744,8 @@ gdict_window_init (GdictWindow *window)
 }
 
 GtkWidget *
-gdict_window_new (GdictSourceLoader *loader,
+gdict_window_new (GdictWindowAction  action,
+		  GdictSourceLoader *loader,
 		  const gchar       *source_name,
 		  const gchar       *word)
 {
@@ -1541,16 +1754,48 @@ gdict_window_new (GdictSourceLoader *loader,
   g_return_val_if_fail (GDICT_IS_SOURCE_LOADER (loader), NULL);
   
   retval = g_object_new (GDICT_TYPE_WINDOW,
+  			 "action", action,
                          "source-loader", loader,
 			 "source-name", source_name,
 			 NULL);
 
   if (word && word[0] != '\0')
     {
-      gtk_entry_set_text (GTK_ENTRY (GDICT_WINDOW (retval)->entry), word);
-      
-      gdict_window_set_word (GDICT_WINDOW (retval), word, NULL);
+      switch (action)
+        {
+	case GDICT_WINDOW_ACTION_LOOKUP:
+	  gtk_entry_set_text (GTK_ENTRY (GDICT_WINDOW (retval)->entry), word);
+	  gdict_window_set_word (GDICT_WINDOW (retval), word, NULL);
+	  break;
+	case GDICT_WINDOW_ACTION_MATCH:
+	  gtk_entry_set_text (GTK_ENTRY (GDICT_WINDOW (retval)->entry), word);
+	  /* FIXME */
+	case GDICT_WINDOW_ACTION_CLEAR:
+	  break;
+	default:
+	  g_assert_not_reached ();
+	  break;
+	}
     }
 
   return retval;
 }
+
+/* GdictWindowAction */
+static const GEnumValue _gdict_window_action_values[] = {
+  { GDICT_WINDOW_ACTION_LOOKUP, "GDICT_WINDOW_ACTION_LOOKUP", "lookup" },
+  { GDICT_WINDOW_ACTION_MATCH, "GDICT_WINDOW_ACTION_MATCH", "match" },
+  { GDICT_WINDOW_ACTION_CLEAR, "GDICT_WINDOW_ACTION_CLEAR", "clear" },
+  { 0, NULL, NULL }
+};
+
+GType
+gdict_window_action_get_type (void)
+{
+  static GType our_type = 0;
+
+  if (!our_type)
+    our_type = g_enum_register_static ("GdictWindowAction", _gdict_window_action_values);
+
+  return our_type;
+}
